'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.hey = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = scuttlebutt;

var _dispatcher = require('./dispatcher');

var _dispatcher2 = _interopRequireDefault(_dispatcher);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Primus = window.Primus;


// Store enhancer
// Wraps createStore to inject our history reducer
function scuttlebutt() {
  var options = arguments.length <= 0 || arguments[0] === undefined ? null : arguments[0];

  return function (createStore) {
    // is it more efficient to store previous states, or replay a bunch of
    // previous actions? (until we have COMMIT checkpointing, the former)
    var history = { state: [] },
        gossip = connectGossip(new _dispatcher2.default());

    return function (reducer, preloadedState, enhancer) {
      var store = createStore(gossip.wrapReducer(reducer), preloadedState, enhancer);

      return _extends({}, store, {
        dispatch: gossip.dispatch(store.dispatch)
      });
    };
  };
}

// initialise network io
function connectGossip(scuttlebutt) {
  var io = Primus.connect('http://localhost:3000');

  console.log('[io] connecting...');

  connectStreams(io, scuttlebutt.createStream());

  return scuttlebutt;
}

// the internet is a series of tubes
function connectStreams(io, gossip) {
  // would love to do this. it doesn't work:
  // spark.pipe(docStream).pipe(spark)

  io.on('connection', function (spark) {
    // spark is the new connection.
    console.log('[io] connected', spark);
  });

  io.on('disconnection', function (spark) {
    // spark is the new disconnection.
    console.log('[io] disconnected', spark);
  });

  io.on('data', function message(data) {
    console.log('[io] <-', data);
    gossip.write(data);
  });

  gossip.on('data', function (data) {
    console.log('[io] ->', data);
    io.write(data);
  });

  // network events

  io.on('open', function () {
    console.log('[io] connection open');
  });

  io.on('error', function (error) {
    console.log('[io] error', error);
  });

  // store stream events

  gossip.on('error', function (error) {
    console.warn('[gossip] error', error);
  });

  // handshake header recieved from a new peer. includes their id and clock info
  gossip.on('header', function (header) {
    var id = header.id;
    var clock = header.clock;

    console.log('[gossip] header', id);
  });
}

// --------------

var hey = exports.hey = function hey() {
  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
  return function (store) {
    return function (next) {
      // store events

      return function (action) {

        var nextState = next(action);

        history.actions.push(action);
        history.state.push(nextState);

        if (!action.__remote) {
          var result = gossip.localUpdate(action);

          // console.log('[gossip] action status', result)
          // console.log('[gossip] state', actions.asArray())
        }

        // return next(action)
        return nextState;
      };
    };
  };
};